@@ -2023,6 +2023,100 @@ SEXP spR_optimize(SEXP x, SEXP verbP)
     return R_NilValue;
 }
 
+#define GHQ_EPS    1e-15
+#define GHQ_MAXIT  40
+
+static int internal_ghq(int N, double *X, double *W)
+{
+    int NR, IT, I, K, J;
+    double Z = 0, HF = 0, HD = 0;
+    double Z0, F0, F1, P, FD, Q, WP, GD, R, R1, R2;
+    double HN = 1/(double)N;
+
+/*   scanf("%d", &N); */
+
+/*   X = malloc( (N + 1) * sizeof(double) ); */
+/*   W = malloc( (N + 1) * sizeof(double) ); */
+
+
+    for(NR = 1; NR <= N / 2; ++NR){
+	if(NR == 1)
+	    Z = -1.1611 + 1.46 * sqrt( (double)N);
+	else
+	    Z -= HN * (N/2 + 1 - NR);
+/* 	IT = 0; */
+/* 	do{ */
+	for (IT = 0; IT <= GHQ_MAXIT; IT++) {
+	    Z0 = Z;
+	    F0 = 1.0;
+	    F1 = 2.0 * Z;
+	    for(K = 2; K <= N; ++K){
+		HF = 2.0 * Z * F1 - 2.0 * (double)(K - 1.0) * F0;
+		HD = 2.0 * K * F1;
+		F0 = F1;
+		F1 = HF;
+	    }
+	    P = 1.0;
+	    for(I = 1; I <= NR-1; ++I){
+		P *= (Z - X[I]);
+	    }
+	    FD = HF / P;
+	    Q = 0.0;
+	    for(I = 1; I <= NR - 1; ++I){
+		WP = 1.0;
+		for(J = 1; J <= NR - 1; ++J){
+		    if(J != I) WP *= ( Z - X[J] );
+		}
+		Q += WP;
+	    }
+	    GD = (HD-Q*FD)/P;
+	    Z -= (FD/GD);
+	    if (abs((Z - Z0) / Z) < GHQ_EPS) break;
+/* 	    IT++; */
+	} /* while(IT <= GHQ_MAXIT && abs((Z-Z0) / Z) > GHQ_EPS); */
+/* 	printf("[%d] %.4f\t", NR, Z ); */
+
+	X[NR] = Z;
+	X[N+1-NR] = -Z;
+	R=1.0;
+	for(K = 1; K <= N; ++K){
+	    R *= (2.0 * (double)K );
+	}
+	W[N+1-NR] = W[NR] = 3.544907701811 * R / (HD*HD);
+    }
+    if( N % 2 ){
+	R1=1.0;
+	R2=1.0;
+	for(J = 1; J <= N; ++J){
+	    R1=2.0*R1*J;
+	    if(J>=(N+1)/2) R2 *= J;
+	}
+	W[N/2+1]=0.88622692545276*R1/(R2*R2);
+	X[N/2+1]=0.0;
+    }
+
+/*   printf("\n"); */
+/*   for(I = 1; I <= N; ++I){ */
+/*     printf("%.4f %.4f\n", X[I], W[I]); */
+/*   } */
+  return 0;
+}
+
+SEXP lme4_ghq(SEXP np)
+{
+    int n = asInteger(np);
+    SEXP ans = PROTECT(allocVector(VECSXP, 2));
+
+    if (n < 1) n = 1;
+    SET_VECTOR_ELT(ans, 0, allocVector(REALSXP, n + 1));
+    SET_VECTOR_ELT(ans, 1, allocVector(REALSXP, n + 1));
+    
+    internal_ghq(n, REAL(VECTOR_ELT(ans, 0)), REAL(VECTOR_ELT(ans, 1)));
+    UNPROTECT(1);
+    return ans;
+}
+
+
 #if 0
 
 /* Gauss-Hermite Quadrature x positions and weights */
@@ -2160,6 +2254,8 @@ static void MCMC_ST(SEXP x, double sigma, double *vals)
     Free(Ptu);
 }
 
+
+
 /* Functions for sampling from a Wishart distribution */
 /* FIXME: Move these to the R sources */
 
